@@ -25,7 +25,6 @@
 
 #include "config.h"
 
-#include <glade/glade.h>
 #include <string.h>
 #include <stdlib.h>
 #include "zenity.h"
@@ -34,7 +33,7 @@
 #define MAX_ELEMENTS_BEFORE_SCROLLING 5
 #define PRINT_HIDE_COLUMN_SEPARATOR ","
 
-static GladeXML *glade_dialog;
+static GtkBuilder *builder;
 static GSList *selected;
 static gchar *separator;
 static gboolean print_all_columns = FALSE;
@@ -94,13 +93,13 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
 {
   static GtkTreeView *tree_view;
   GtkTreeModel *model;
-  GtkTreeIter iter;
+  static GtkTreeIter iter;
   static gint column_count = 0;
   static gint row_count = 0;
   static gint n_columns;
   static gboolean editable;
   static gboolean toggles;
-  static gboolean first_time = FALSE;
+  static gboolean first_time = TRUE;
 
   tree_view = GTK_TREE_VIEW (data);
   n_columns = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tree_view), "n_columns"));
@@ -109,8 +108,8 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
 
   model = gtk_tree_view_get_model (tree_view);
 
-  if (!first_time) {
-    first_time = TRUE;
+  if (first_time) {
+    first_time = FALSE;
     gtk_list_store_append (GTK_LIST_STORE (model), &iter);
   }
 
@@ -150,7 +149,7 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
       }
 
       if (toggles && column_count == 0) {
-        if (strcmp (g_strdown (zenity_util_strip_newline (string->str)), "true") == 0)
+        if (strcmp (g_ascii_strdown (zenity_util_strip_newline (string->str), -1), "true") == 0)
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, TRUE, -1);
         else
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, FALSE, -1);
@@ -159,28 +158,29 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
         gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, zenity_util_strip_newline (string->str), -1);        
       }
 
-    if (editable) {
-      gtk_list_store_set (GTK_LIST_STORE (model), &iter, n_columns, TRUE, -1);
-    }
+      if (editable) {
+        gtk_list_store_set (GTK_LIST_STORE (model), &iter, n_columns, TRUE, -1);
+      }
 
-    if (row_count == MAX_ELEMENTS_BEFORE_SCROLLING) {
-      GtkWidget *scrolled_window;
-      GtkRequisition rectangle;
+      if (row_count == MAX_ELEMENTS_BEFORE_SCROLLING) {
+        GtkWidget *scrolled_window;
+        GtkRequisition rectangle;
 
-      gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
-      scrolled_window = glade_xml_get_widget (glade_dialog, "zenity_tree_window");
-      gtk_widget_set_size_request (scrolled_window, -1, rectangle.height);
-      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-                                      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    }
+        gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
+        scrolled_window = GTK_WIDGET (gtk_builder_get_object (builder,
+							 "zenity_tree_window"));
+        gtk_widget_set_size_request (scrolled_window, -1, rectangle.height);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+                                        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+      }
 
-    column_count ++;
+      column_count++;
 
     } while (g_io_channel_get_buffer_condition (channel) == G_IO_IN); 
     g_string_free (string, TRUE);
   }
 
-  if (condition != G_IO_IN) {
+  if ((condition != G_IO_IN) && (condition != G_IO_IN + G_IO_HUP)) {
     g_io_channel_shutdown (channel, TRUE, NULL);
     return FALSE;
   }
@@ -228,7 +228,7 @@ zenity_tree_fill_entries (GtkTreeView  *tree_view,
     for (j = 0; j < n_columns; j++) {
         
       if (toggles && j == 0) {
-        if (strcmp (g_strdown ((gchar *) args[i+j]), "true") == 0)
+        if (strcmp (g_ascii_strdown ((gchar *) args[i+j], -1), "true") == 0)
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, j, TRUE, -1);
         else 
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, j, FALSE, -1);
@@ -245,7 +245,8 @@ zenity_tree_fill_entries (GtkTreeView  *tree_view,
       GtkRequisition rectangle;
 
       gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
-      scrolled_window = glade_xml_get_widget (glade_dialog, "zenity_tree_window");
+      scrolled_window = GTK_WIDGET (gtk_builder_get_object (builder,
+      							 "zenity_tree_window"));
       gtk_widget_set_size_request (scrolled_window, -1, rectangle.height);
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -281,8 +282,8 @@ void
 zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
 {
   GtkWidget *dialog;
-  GtkWidget *tree_view;
-  GtkWidget *text;
+  GObject *tree_view;
+  GObject *text;
   GtkTreeViewColumn *column;
   GtkListStore *model;
   GType *column_types;
@@ -290,26 +291,26 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
   gboolean first_column = FALSE;
   gint i, column_index, n_columns;
 
-  glade_dialog = zenity_util_load_glade_file ("zenity_tree_dialog");
+  builder = zenity_util_load_ui_file ("zenity_tree_dialog", NULL);
 
-  if (glade_dialog == NULL) {
+  if (builder == NULL) {
     data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
     return;
   }
         
-  separator = g_strdup (tree_data->separator);
+  separator = g_strcompress (tree_data->separator);
 
   n_columns = g_slist_length (tree_data->columns);
 
   if (tree_data->print_column) {
-    if (strcmp (g_strdown (tree_data->print_column), "all") == 0)
+    if (strcmp (g_ascii_strdown (tree_data->print_column, -1), "all") == 0)
       print_all_columns = TRUE;
     else 
       print_columns = zenity_tree_extract_column_indexes (tree_data->print_column, n_columns);
   }
   else { 
     print_columns = g_new (gint, 2);
-    print_columns[0] = 1;
+    print_columns[0] = (tree_data->radiobox || tree_data->checkbox ? 2 : 1);
     print_columns[1] = 0;
   }
 
@@ -328,9 +329,9 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
     return;
   }
 
-  glade_xml_signal_autoconnect (glade_dialog);
+  gtk_builder_connect_signals (builder, NULL);
 
-  dialog = glade_xml_get_widget (glade_dialog, "zenity_tree_dialog");
+  dialog = GTK_WIDGET (gtk_builder_get_object (builder, "zenity_tree_dialog"));
 
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (zenity_tree_dialog_response), data);
@@ -338,7 +339,7 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
   if (data->dialog_title)
     gtk_window_set_title (GTK_WINDOW (dialog), data->dialog_title);
 
-  text = glade_xml_get_widget (glade_dialog, "zenity_tree_text");
+  text = gtk_builder_get_object (builder, "zenity_tree_text");
                                                                                 
   if (tree_data->dialog_text)
     gtk_label_set_markup (GTK_LABEL (text), g_strcompress (tree_data->dialog_text));
@@ -348,10 +349,10 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
   if (data->width > -1 || data->height > -1)
     gtk_window_set_default_size (GTK_WINDOW (dialog), data->width, data->height);
 
-  tree_view = glade_xml_get_widget (glade_dialog, "zenity_tree_view");
+  tree_view = gtk_builder_get_object (builder, "zenity_tree_view");
 
   if (!(tree_data->radiobox || tree_data->checkbox)) 
-    g_signal_connect (G_OBJECT (tree_view), "row-activated", 
+    g_signal_connect (tree_view, "row-activated", 
                       G_CALLBACK (zenity_tree_row_activated), data);
  
   /* Create an empty list store */
@@ -479,6 +480,9 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
 
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), TRUE);
 
+  if (tree_data->hide_header)
+    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree_view), FALSE);
+
   if (tree_data->radiobox || tree_data->checkbox) {
     if (tree_data->data && *tree_data->data)
       zenity_tree_fill_entries (GTK_TREE_VIEW (tree_view), tree_data->data, n_columns, TRUE, tree_data->editable);
@@ -493,10 +497,14 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
   }
 
   zenity_util_show_dialog (dialog);
+
+  if(data->timeout_delay > 0) {
+    g_timeout_add_seconds (data->timeout_delay, (GSourceFunc) zenity_util_timeout_handle, NULL);
+  }
+
   gtk_main ();
 
-  if (glade_dialog)
-    g_object_unref (glade_dialog);
+  g_object_unref (builder);
 }
 
 static void 
@@ -511,7 +519,7 @@ zenity_tree_dialog_get_selected (GtkTreeModel *model, GtkTreePath *path_buf, Gtk
     for (i = 0; i < n_columns; i++) {
       gtk_tree_model_get_value (model, iter, i, &value);
 
-      selected = g_slist_append (selected, g_strdup (g_value_get_string (&value)));
+      selected = g_slist_append (selected, g_value_dup_string (&value));
       g_value_unset (&value);
     }
     return;
@@ -520,7 +528,7 @@ zenity_tree_dialog_get_selected (GtkTreeModel *model, GtkTreePath *path_buf, Gtk
   for (i = 0; print_columns[i] != 0; i++) {
     gtk_tree_model_get_value  (model, iter, print_columns[i] - 1, &value);
 
-    selected = g_slist_append (selected, g_strdup (g_value_get_string (&value)));
+    selected = g_slist_append (selected, g_value_dup_string (&value));
     g_value_unset (&value);
   }
 }
@@ -542,7 +550,7 @@ zenity_tree_dialog_toggle_get_selected (GtkTreeModel *model, GtkTreePath *path,
       for (i = 1; i < n_columns; i++) {
         gtk_tree_model_get_value (model, iter, i, &value);
         
-        selected = g_slist_append (selected, g_strdup (g_value_get_string (&value)));
+        selected = g_slist_append (selected, g_value_dup_string (&value));
         g_value_unset (&value);
       }
       g_value_unset (&toggle_value);
@@ -552,7 +560,7 @@ zenity_tree_dialog_toggle_get_selected (GtkTreeModel *model, GtkTreePath *path,
     for (i = 0; print_columns[i] != 0; i++) {
       gtk_tree_model_get_value (model, iter, print_columns[i] - 1, &value);
 
-      selected = g_slist_append (selected, g_strdup (g_value_get_string (&value)));
+      selected = g_slist_append (selected, g_value_dup_string (&value));
       g_value_unset (&value);
     }
   }
@@ -586,13 +594,13 @@ static void
 zenity_tree_dialog_response (GtkWidget *widget, int response, gpointer data)
 {
   ZenityData *zen_data = data;
-  GtkWidget *tree_view;
+  GObject *tree_view;
   GtkTreeSelection *selection; 
   GtkTreeModel *model;
 
   switch (response) {
     case GTK_RESPONSE_OK:
-      tree_view = glade_xml_get_widget (glade_dialog, "zenity_tree_view");
+      tree_view = gtk_builder_get_object (builder, "zenity_tree_view");
       model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
 
       if (gtk_tree_model_get_column_type (model, 0) == G_TYPE_BOOLEAN)
